var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { registerWebPlugin, WebPlugin } from '@capacitor/core';
export class JokHelperWeb extends WebPlugin {
    constructor() {
        super({
            name: 'JokHelper',
            platforms: ['web'],
        });
        this.audioCache = new Map();
    }
    echo(options) {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('ECHO', options);
            return options;
        });
    }
    setKeychainItem({ key, value }) {
        return __awaiter(this, void 0, void 0, function* () {
            localStorage.setItem(key, value);
            return { value };
        });
    }
    getKeychainItem({ key }) {
        return __awaiter(this, void 0, void 0, function* () {
            const value = localStorage.getItem(key);
            return { value };
        });
    }
    setDeviceOrientationLock({ orientationMask, }) {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('setOrientationLock', orientationMask);
        });
    }
    getDeviceOrientation() {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('getDeviceOrientation');
            return null;
        });
    }
    listenDeviceOrientationChanges() {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('listenDeviceOrientationChanges');
        });
    }
    listenPushNotificationEvents() {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('listenPushNotificationEvents');
        });
    }
    isWideScreen() {
        return Promise.resolve({ value: false });
    }
    isMobileDevice() {
        return Promise.resolve({ value: false });
    }
    getPushNotificationsState() {
        return Promise.resolve(null);
    }
    askPushNotificationsPermission() {
        return Promise.resolve({ accepted: false });
    }
    openAppSettings() {
        return Promise.resolve({ success: false });
    }
    canMakePayments() {
        return Promise.resolve({ value: false });
    }
    loadProducts(_) {
        return Promise.resolve({
            success: false,
            products: [],
            invalidProducts: [],
        });
    }
    requestPayment(_) {
        return Promise.resolve({
            success: false,
            message: '',
        });
    }
    finishPayment(_) {
        return Promise.resolve({
            success: false,
            message: '',
        });
    }
    listenTransactionStateChanges() {
        return Promise.resolve(null);
    }
    platformInfo() {
        return Promise.resolve({
            success: true,
            platform: 'WEB',
            clientVersion: '',
        });
    }
    viewAppPage(_data) {
        return Promise.resolve({ value: false });
    }
    playAudio(data) {
        const { name, path = '/assets/audios' } = data;
        const url = `${path}/${name}.mp3`;
        if (!this.audioCache.has(url)) {
            this.audioCache.set(url, new Audio(url));
        }
        const audioElement = this.audioCache.get(url);
        if (!audioElement) {
            return Promise.resolve({ value: false });
        }
        audioElement.currentTime = 0;
        audioElement.play();
        return Promise.resolve({ value: true });
    }
    openMailbox() {
        return Promise.resolve({ value: false });
    }
    vibrate(pattern) {
        if (!navigator || !navigator.vibrate) {
            return Promise.resolve({ value: false });
        }
        navigator.vibrate(pattern);
        return Promise.resolve({ value: true });
    }
    requestReview() {
        return Promise.resolve({ value: true });
    }
    configureRewardedAds(_) {
        return Promise.resolve({ success: false });
    }
    listenRewardedAdsWatchedEvents() {
        return Promise.resolve({ value: false });
    }
    showRewardedAds(_) {
        return Promise.resolve({
            started: false,
            additionalInfo: 'Not implemented',
            errorCode: '',
            errorMessage: '',
        });
    }
}
const JokHelper = new JokHelperWeb();
export { JokHelper };
registerWebPlugin(JokHelper);
//# sourceMappingURL=web.js.map